/***************************************************************************
 * MoioChart Library - graphs and charts made easy                         *
 * Copyright (C) 2005 by Silvio Moioli                                     *
 * silvio@moioli.net, http://www.moioli.net                                *  
 *                                                                         *
 * This program is free software; you can redistribute it and/or           *
 * modify it under the terms of the GNU General Public License             *
 * as published by the Free Software Foundation; either version 2          *
 * of the License, or (at your option) any later version.                  *
 *                                                                         *
 * This program is distributed in the hope that it will be useful,         *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of          *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           *
 * GNU General Public License for more details.                            *
 *                                                                         *
 * You should have received a copy of the GNU General Public License       *
 * along with this program; if not, write to the Free Software             *
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,              *
 * MA  02110-1301, USA.                                                    *
 ***************************************************************************/

package net.moioli.chart.sw;

import moio.util.Collection;
import moio.util.Iterator;

import net.moioli.chart.ChartI;
import net.moioli.chart.DefaultChart;
import net.moioli.chart.IntRectangle;

import waba.fx.Color;
import waba.fx.Graphics;
import waba.fx.Image;
import waba.fx.Rect;
import waba.sys.Settings;
import waba.ui.Control;
import waba.ui.Label;
import waba.ui.MainWindow;
import waba.ui.Window;

/**
 * @author silvio
 *
 * TODO To change the template for this generated type comment go to
 * Window - Preferences - Java - Code Style - Code Templates
 */
public class SWGraph extends Control implements Collection{
    
    //Il grafico da disegnare
    private DefaultChart c = null;
    /** Precisione di default (media) */
    public static final int PRECISIONE = 4;
    
    /**
     * Costruisce un grafico vuoto.
     */
    public SWGraph(){
        setGraph(new DefaultChart()); 
        this.c.setPixelAccuracy(PRECISIONE);
    }
        
    /**
     * Costruisce un grafico con piï¿½ oggetti. 
     * @param    c    DefaultChart da rappresentare
     */
    public SWGraph(DefaultChart g){
        setGraph(g);
        g.setMinX(-10);
        g.setMaxX(10);
        g.setPixelAccuracy(PRECISIONE);
    }
    
    public void onPaint(Graphics g) {
        //Ridisegno solo se c'è qualcosa nel grafico, evito di
        //disegnare solo gli assi e il logo.
        if (this.c.size() > 0){
            //WaitWindow ww = new WaitWindow();
            //MainWindow.getMainWindow().popupModal(ww);
            Rect r = getRect();
            Image buf = new Image(r.width, r.height);
	        Graphics gb = buf.getGraphics();
	        if ((r.width > 30) && (r.height > 30)){
	            IntRectangle r2 = new IntRectangle(r.x,r.y,r.width,r.height);
	            drawMe(gb,r2);
	        }
	        //ww.unpop();
	        g.drawImage(buf,0,0);
        }
    }
    
    public Object clone(){
        return new SWGraph(this.c);
    }
    
    /**
     * @return Returns the grafico.
     */
    public ChartI getGraph() {
        return this.c;
    }

    /**
     * @param c The grafico to set.
     */
    public void setGraph(DefaultChart g) {
        this.c = g;
        repaintNow();
    }


    /**
     * Decorates Collection.
     */
    public boolean add(Object arg0) {
    	boolean result = this.c.add(arg0);
        repaintNow();
        return result;
    }

    /**
     * Decorates Collection.
     */
    public boolean addAll(Collection arg0) {
    	boolean result = this.c.addAll(arg0);
    	repaintNow();
    	return result;
    }

    /**
     * Decorates Collection.
     */
    public void clear() {
        this.c.clear();
    	repaintNow();
    }

    /**
     * Decorates Collection.
     */
    public boolean contains(Object arg0) {
        return this.c.contains(arg0);
    }

    /**
     * Decorates Collection.
     */
    public boolean containsAll(Collection arg0) {
        return this.c.containsAll(arg0);
    }

    /**
     * Decorates Collection.
     */
    public void drawMe(Graphics g, IntRectangle clip) {
            g.setForeColor(Color.defaultBackColor);
            g.setBackColor(Color.defaultBackColor);
            //HACK here
            g.fillRect(clip.x-1,clip.y-1,clip.width,clip.height);
            this.c.drawMe(new SuperWabaGraphicsWrapper(g), clip);
    }

    /**
     * Decorates Collection.
     */
    public double getMaxX() {
        return this.c.getMaxX();
    }

    /**
     * Decorates Collection.
     */
    public double getMinX() {
        return this.c.getMinX();
    }
    
    /**
     * Decorates Collection.
     */
    public double getMaxY() {
        return this.c.getMaxY();
    }

    /**
     * Decorates Collection.
     */
    public double getMinY() {
        return this.c.getMinY();
    }
    
    /**
     * Decorates Collection.
     */
    public int getPrecisionePixel() {
        return this.c.getPixelAccuracy();
    }

    /**
     * Decorates Collection.
     */
    public boolean isEmpty() {
        return this.c.isEmpty();
    }

    /**
     * Decorates Collection.
     */
    public Iterator iterator() {
        return this.c.iterator();
    }

    /**
     * Decorates Collection.
     */
    public boolean remove(Object arg0) {
    	boolean result = this.c.remove(arg0);
        repaintNow();
    	return result;
    }

    /**
     * Decorates Collection.
     */
    public boolean removeAll(Collection arg0) {
    	boolean result = this.c.removeAll(arg0);
    	repaintNow();
    	return result;
    }

    /**
     * Decorates Collection.
     */
    public boolean retainAll(Collection arg0) {
    	boolean result = this.c.retainAll(arg0);
    	repaintNow();
    	return result;
    }

    /**
     * Decorates Collection.
     */
    public void setMaxX(double maxX) {
        this.c.setMaxX(maxX);
    	//repaintNow();
    }

    /**
     * Decorates Collection.
     */
    public void setMinX(double minX) {
        this.c.setMinX(minX);
    	//repaintNow();
    }
    
    /**
     * Decorates Collection.
     */
    public void setMaxY(double maxY) {
        this.c.setMaxY(maxY);
    	//repaintNow();
    }

    /**
     * Decorates Collection.
     */
    public void setMinY(double minY) {
        this.c.setMinY(minY);
    	//repaintNow();
    }

    /**
     * Decorates Collection.
     */
    public void setPrecisionePixel(int precisionePixel) {
        this.c.setPixelAccuracy(precisionePixel);
    	repaintNow();
    }

    /**
     * Decorates Collection.
     */
    public int size() {
    	return this.c.size();
    }

    /**
     * Decorates Collection.
     */
    public Object[] toArray() {
        return this.c.toArray();
    }

    /**
     * Decorates Collection.
     */
    public Object[] toArray(Object[] arg0) {
        return this.c.toArray(arg0);
    }
   
    private class WaitWindow extends Window{
        Label l = new Label("Wait please...");
        WaitWindow(){
            super("Test",RECT_BORDER); // with caption and borders
            super.setRect(CENTER,CENTER,Settings.screenWidth/2,Settings.screenHeight/4);
            super.add(this.l,CENTER,CENTER);
        }
    }
}
